
--
-- Name: Vario
-- Description: Variable transmission implementation for steerables
-- Version: 1.0
-- Author: Saracaen & Face
-- Date: 24/07/2011
--

Vario = {};

Vario.modDirectory = g_currentModDirectory;

-- Extended original functions --
function Vario:load(xmlFile)

	local check1 = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.features#vario"), true);
	self.isVarioEnabled = check1;
		
	if self.customEnvironment ~= nil then
		local specialization1 = loadstring(string.format("return %s.ESLimiter", self.customEnvironment))();
		local specialization2 = loadstring(string.format("return %s.rpmlimiter", self.customEnvironment))();
		
		local check2 = SpecializationUtil.hasSpecialization(specialization1, self.specializations);
		local check3 = SpecializationUtil.hasSpecialization(specialization2, self.specializations);
				
		self.isVarioEnabled = check1 and not check2 and not check3;
	end;
			
	if self.isVarioEnabled then
		self.setVarioLimit = Vario.setVarioLimit;
	
		self.maxRpm = {};
		for speedLevel, maxRpm in pairs(self.motor.maxRpm) do
			self.maxRpm[speedLevel] = maxRpm;
		end;
		
		self.percentages = {};
		for speedLevel, _ in pairs(self.motor.maxRpm) do
			self.percentages[speedLevel] = 100;
		end;
		
		self.speedLevel = 3;
		self.synchronized = true;
		
		self.varioOverlay = Overlay:new("varioOverlay", Utils.getFilename("overlays/vario_hud.png", Vario.modDirectory), 0.7543, 0.1715, 0.2371, 0.039525);
	end;

end;

Steerable.load = Utils.appendedFunction(Steerable.load, Vario.load);

function Vario:readStream(streamId, connection)

	if self.isVarioEnabled then
		for speedLevel, _ in pairs(self.percentages) do
			local percentage = streamReadInt8(streamId);
			self:setVarioLimit(percentage, speedLevel, true);
		end;
	end;

end;

Steerable.readStream = Utils.appendedFunction(Steerable.readStream, Vario.readStream);

function Vario:writeStream(streamId, connection)

	if self.isVarioEnabled then
		for _, percentage in pairs(self.percentages) do
			streamWriteInt8(streamId, percentage);
		end;
	end;

end;

Steerable.writeStream = Utils.appendedFunction(Steerable.writeStream, Vario.writeStream);

function Vario:update(dt)
	
	if self.isVarioEnabled then
		self.speedLevel = 3;
		if self.motor.speedLevel ~= 0 then
			self.speedLevel = self.motor.speedLevel;
		end;
	
		if self:getIsActiveForInput() then
			if InputBinding.isPressed(InputBinding.VARIO_UP) then
				self:setVarioLimit(math.min(self.percentages[self.speedLevel] + 1, 100), self.speedLevel, true);
				self.synchronized = false;
			elseif InputBinding.isPressed(InputBinding.VARIO_DOWN) then
				self:setVarioLimit(math.max(self.percentages[self.speedLevel] - 1, 1), self.speedLevel, true);
				self.synchronized = false;
			elseif not self.synchronized then
				self:setVarioLimit(self.percentages[self.speedLevel], self.speedLevel);
				self.synchronized = true;
			end;
		end;
	end;
	
end;

Steerable.update = Utils.appendedFunction(Steerable.update, Vario.update);

function Vario:draw()

	if self.isVarioEnabled then
		if self.isClient then
			local percentage = self.percentages[self.speedLevel];
			if percentage < 100 then
				percentage = " " .. percentage;
			end;
			
			local text = g_i18n:getText("Vario_level") .. self.speedLevel .. ":" .. percentage .. "% ";
			
			self.varioOverlay:render();
			setTextBold(true);
			setTextAlignment(RenderText.ALIGN_LEFT);
			renderText(0.7543 + 0.13, 0.1715 + 0.0085, 0.02, text);
	
			g_currentMission:addExtraPrintText(string.format(g_i18n:getText("Vario_controls"), self.typeDesc) .. " " .. InputBinding.getKeyNamesOfDigitalAction(InputBinding.VARIO_UP) .. "/" .. InputBinding.getKeyNamesOfDigitalAction(InputBinding.VARIO_DOWN));
		end;
	end;

end;

Steerable.draw = Utils.appendedFunction(Steerable.draw, Vario.draw);

-- New functions --
function Vario:setVarioLimit(percentage, speedLevel, noEventSend)
	
	SetVarioLimitEvent.sendEvent(self, percentage, speedLevel, noEventSend)
	self.percentages[speedLevel] = percentage;
	self.motor.maxRpm[speedLevel] = (percentage / 100) * self.maxRpm[speedLevel];
	
end;